@@ -41,10 +41,6 @@ GRL_LOG_DOMAIN_STATIC(local_metadata_log_domain);
 #define SOURCE_NAME "Local Metadata Provider"
 #define SOURCE_DESC "A source providing locally available metadata"
 
-#define AUTHOR      "Igalia S.L."
-#define LICENSE     "LGPL"
-#define SITE        "http://www.igalia.com"
-
 /**/
 
 #define TV_REGEX                                \
@@ -115,6 +111,9 @@ static gboolean grl_local_metadata_source_may_resolve (GrlMetadataSource *source
                                                        GrlKeyID key_id,
                                                        GList **missing_keys);
 
+static void grl_local_metadata_source_cancel (GrlMetadataSource *source,
+                                              guint operation_id);
+
 gboolean grl_local_metadata_source_plugin_init (GrlPluginRegistry *registry,
                                                const GrlPluginInfo *plugin,
                                                GList *configs);
@@ -137,11 +136,11 @@ grl_local_metadata_source_plugin_init (GrlPluginRegistry *registry,
   GRL_DEBUG ("grl_local_metadata_source_plugin_init");
 
   if (!configs) {
-    GRL_WARNING ("\tConfiguration not provided! Using default configuration.");
+    GRL_INFO ("\tConfiguration not provided! Using default configuration.");
   } else {
     config_count = g_list_length (configs);
     if (config_count > 1) {
-      GRL_WARNING ("\tProvided %i configs, but will only use one", config_count);
+      GRL_INFO ("\tProvided %i configs, but will only use one", config_count);
     }
 
     config = GRL_CONFIG (configs->data);
@@ -184,6 +183,7 @@ grl_local_metadata_source_class_init (GrlLocalMetadataSourceClass * klass)
   metadata_class->supported_keys = grl_local_metadata_source_supported_keys;
   metadata_class->may_resolve = grl_local_metadata_source_may_resolve;
   metadata_class->resolve = grl_local_metadata_source_resolve;
+  metadata_class->cancel = grl_local_metadata_source_cancel;
 
   g_class->set_property = grl_local_metadata_source_set_property;
 
@@ -442,11 +442,11 @@ got_file_info (GFile *file, GAsyncResult *result,
     grl_media_set_thumbnail (rs->media, thumbnail_uri);
     g_free (thumbnail_uri);
 
-    rs->callback (rs->source, rs->media, rs->user_data, NULL);
+    rs->callback (rs->source, rs->resolve_id, rs->media, rs->user_data, NULL);
   } else {
     GRL_INFO ("Could not find thumbnail for media: %s",
               grl_media_get_url (rs->media));
-    rs->callback (rs->source, rs->media, rs->user_data, NULL);
+    rs->callback (rs->source, rs->resolve_id, rs->media, rs->user_data, NULL);
   }
 
   goto exit;
@@ -455,7 +455,7 @@ error:
     {
       GError *new_error = g_error_new (GRL_CORE_ERROR, GRL_CORE_ERROR_RESOLVE_FAILED,
                                        "Got error: %s", error->message);
-      rs->callback (rs->source, rs->media, rs->user_data, new_error);
+      rs->callback (rs->source, rs->resolve_id, rs->media, rs->user_data, new_error);
 
       g_error_free (error);
       g_error_free (new_error);
@@ -467,7 +467,9 @@ exit:
 }
 
 static void
-resolve_video (GrlMetadataSourceResolveSpec *rs, resolution_flags_t flags)
+resolve_video (GrlMetadataSource *source,
+               GrlMetadataSourceResolveSpec *rs,
+               resolution_flags_t flags)
 {
   gchar *title, *showname;
   GDateTime *date;
@@ -543,23 +545,30 @@ resolve_video (GrlMetadataSourceResolveSpec *rs, resolution_flags_t flags)
 }
 
 static void
-resolve_image (GrlMetadataSourceResolveSpec *rs, resolution_flags_t flags)
+resolve_image (GrlMetadataSource *source,
+               GrlMetadataSourceResolveSpec *rs,
+               resolution_flags_t flags)
 {
   GFile *file;
+  GCancellable *cancellable;
 
   GRL_DEBUG ("resolve_image");
 
   if (flags & FLAG_THUMBNAIL) {
     file = g_file_new_for_uri (grl_media_get_url (rs->media));
 
+    cancellable = g_cancellable_new ();
+    grl_metadata_source_set_operation_data (source, rs->resolve_id, cancellable);
     g_file_query_info_async (file, G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
-                             G_FILE_QUERY_INFO_NONE, G_PRIORITY_DEFAULT, NULL,
+                             G_FILE_QUERY_INFO_NONE, G_PRIORITY_DEFAULT, cancellable,
                              (GAsyncReadyCallback)got_file_info, rs);
   }
 }
 
 static void
-resolve_album_art (GrlMetadataSourceResolveSpec *rs, resolution_flags_t flags)
+resolve_album_art (GrlMetadataSource *source,
+                   GrlMetadataSourceResolveSpec *rs,
+                   resolution_flags_t flags)
 {
   /* FIXME: implement this, according to
    * http://live.gnome.org/MediaArtStorageSpec
@@ -570,7 +579,7 @@ resolve_album_art (GrlMetadataSourceResolveSpec *rs, resolution_flags_t flags)
   GError *error;
   error = g_error_new (GRL_CORE_ERROR, GRL_CORE_ERROR_RESOLVE_FAILED,
     "Thumbnail resolution for GrlMediaAudio not implemented in local-metadata");
-  rs->callback (rs->source, rs->media, rs->user_data, error);
+  rs->callback (rs->source, rs->resolve_id, rs->media, rs->user_data, error);
   g_error_free (error);
 }
 
@@ -696,7 +705,7 @@ grl_local_metadata_source_resolve (GrlMetadataSource *source,
 
   if (error) {
     /* No can do! */
-    rs->callback (source, rs->media, rs->user_data, error);
+    rs->callback (source, rs->resolve_id, rs->media, rs->user_data, error);
     g_error_free (error);
     return;
   }
@@ -705,15 +714,27 @@ grl_local_metadata_source_resolve (GrlMetadataSource *source,
 
   if (GRL_IS_MEDIA_VIDEO (rs->media)) {
     if (priv->guess_video)
-      resolve_video (rs, flags);
-    resolve_image (rs, flags);
+      resolve_video (source, rs, flags);
+    resolve_image (source, rs, flags);
   } else if (GRL_IS_MEDIA_IMAGE (rs->media)) {
-    resolve_image (rs, flags);
+    resolve_image (source, rs, flags);
   } else if (GRL_IS_MEDIA_AUDIO (rs->media)) {
-    resolve_album_art (rs, flags);
+    resolve_album_art (source, rs, flags);
   } else {
     /* What's that media type? */
-    rs->callback (source, rs->media, rs->user_data, NULL);
+    rs->callback (source, rs->resolve_id, rs->media, rs->user_data, NULL);
   }
 }
 
+static void
+grl_local_metadata_source_cancel (GrlMetadataSource *source,
+                                  guint operation_id)
+{
+  GCancellable *cancellable =
+    (GCancellable *) grl_metadata_source_get_operation_data (source,
+                                                             operation_id);
+
+  if (cancellable) {
+    g_cancellable_cancel (cancellable);
+  }
+}
